import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/session.dart';
import 'package:livelike_flutter_sdk/src/sponsor.dart';

import 'constants.dart';
import 'model/leaderboard.dart';

class EngagementSDK {
  EngagementSDK(this.clientId) {
    _initSDK(clientId, false, null, null);
  }

  EngagementSDK.accessToken(this.clientId, String? accessToken,
      void Function(String accessToken)? onReceiveAccessToken) {
    _initSDK(clientId, true, accessToken, onReceiveAccessToken);
  }

  final String clientId;

  Stream get errorStream => errorEventChannel.receiveBroadcastStream();

  Sponsor get sponsor => Sponsor();

  Future<void> _initSDK(
      String clientId,
      bool usingAccessToken,
      String? accessToken,
      void Function(String accessToken)? onReceiveAccessToken) async {
    await livelike_method_channel.invokeMethod(initSDKKey, {
      clientIDKey: clientId,
      accessTokenKey: accessToken,
      usingAccessTokenKey: usingAccessToken
    });
    if (usingAccessToken) {
      Stream _accessTokenStream =
          accessTokenEventChannel.receiveBroadcastStream();
      _accessTokenStream.listen((event) {
        onReceiveAccessToken?.call(event);
      });
    }
  }

  Future<Session> createSession(String programId) async {
    Map result = await (livelike_method_channel
        .invokeMethod(createSessionKey, {programIDKey: programId}));
    return Session(result[programIDKey], result[chatRoomIdKey]);
  }

  Future<ChatSession> createChatSession(String chatRoomId) async {
    await livelike_method_channel
        .invokeMethod(createChatSessionKey, {chatRoomIdKey: chatRoomId});
    return ChatSession(chatRoomId);
  }

  Future<List<LeaderBoard>> getLeaderBoards(String programId) async {
    List result = await livelike_method_channel
        .invokeMethod(getLeaderBoardsKey, {programIDKey: programId});
    return result.map((e) => LeaderBoard.fromJson(e)).toList();
  }

  Future<LeaderBoard> getLeaderBoardDetails(String leaderBoardId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardDetailsKey, {leaderBoardIdKey: leaderBoardId});
    return LeaderBoard.fromJson(result);
  }

  Future<LeaderBoardEntry> getLeaderBoardEntryForProfile(
      String leaderBoardId, String profileId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardEntryForProfileKey,
        {leaderBoardIdKey: leaderBoardId, profileIdKey: profileId});
    return LeaderBoardEntry.fromJson(result);
  }

  Future<LeaderBoardEntry> getLeaderBoardEntryForCurrentUserProfile(
      String leaderBoardId) async {
    Map result = await livelike_method_channel.invokeMethod(
        getLeaderBoardEntryForCurrentUserProfileKey,
        {leaderBoardIdKey: leaderBoardId});
    return LeaderBoardEntry.fromJson(result);
  }

  Future<LeaderBoardEntryResult> getEntriesForLeaderBoard(
      String leaderBoardId, LiveLikePagination likePagination) async {
    Map result = await livelike_method_channel
        .invokeMethod(getEntriesForLeaderBoardKey, {
      leaderBoardIdKey: leaderBoardId,
      paginationKey: likePagination.toString().split(".")[1]
    });
    return LeaderBoardEntryResult.fromJson(result);
  }
}
